using System;
using System.Xml;

using Microsoft.Web.Services3.Xml;

namespace WSCompression
{
    internal class CompressedData : IXmlElement
    {
        private string _id;
        private CompressionMode _compressionMode;
        private CompressionLevel _compressionLevel;

        public CompressedData( CompressionMode compressionMode, CompressionLevel compressionLevel, string id )
        {
            _compressionMode = compressionMode;
            _compressionLevel = compressionLevel;
            _id = id;
        }

        public CompressedData( CompressionMode compressionMode, CompressionLevel compressionLevel, XmlElement element )
        {
            _compressionLevel = compressionLevel;
            _compressionMode = compressionMode;
            LoadXml( element );
        }

        public static bool IsValidElement( XmlElement element )
        {
            return ( element.LocalName == WSCompression.ElementNames.CompressedData && element.NamespaceURI == WSCompression.NamespaceURI );
        }

        public XmlElement GetCompressedXml( XmlDocument document, XmlElement body )
        {
            XmlElement compressedData = GetXml( document );
            compressedData.InnerText = CompressAndBase64Encode( body.InnerXml, _compressionMode, _compressionLevel );
            return compressedData;
        }

        public string Decompress( XmlElement compressedElement )
        {
            return Base64DecodeAndDecompress( compressedElement.InnerText, _compressionMode, _compressionLevel );
        }

        private static string CompressAndBase64Encode( string data, CompressionMode compressionMode, CompressionLevel compressionLevel )
        {
            return Convert.ToBase64String( ( new Compressor( compressionMode, compressionLevel ) ).CompressString( data ) );
        }

        private static string Base64DecodeAndDecompress( string compressedData, CompressionMode compressionMode, CompressionLevel compressionLevel )
        {
            return ( new Compressor( compressionMode, compressionLevel ) ).DecompressString( Convert.FromBase64String( compressedData ) );
        }

        #region IXmlElement Members

        public void LoadXml( XmlElement element )
        {
            if ( element == null )
                throw new ArgumentNullException( "element" );

            if ( !IsValidElement( element ) )
                throw new ArgumentException( "element" );

            if ( element.HasAttribute( WSCompression.AttributeNames.Id ) )
                _id = element.GetAttribute( WSCompression.AttributeNames.Id );
        }

        public XmlElement GetXml( XmlDocument document )
        {
            if ( document == null )
                throw new ArgumentNullException( "document" );

            XmlElement dataElement = document.CreateElement( WSCompression.Prefix, WSCompression.ElementNames.CompressedData, WSCompression.NamespaceURI );
            dataElement.SetAttribute( WSCompression.AttributeNames.Id, _id );

            return dataElement;
        }

        #endregion
    }
}