using System;
using System.Collections.Generic;
using System.Globalization;

using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;

namespace WSCompression
{
    public class CompressionAssertion : PolicyAssertion
    {
        private CompressionMode _compressionMode;
        private CompressionLevel _compressionLevel;
        private int _threshold;

        public CompressionAssertion( ) { }
        
        public CompressionAssertion( CompressionMode compressionMode, CompressionLevel compressionLevel, int threshold )
        {
            _compressionMode = compressionMode;
            _compressionLevel = compressionLevel;
            _threshold = threshold;
        }
        
        public override void ReadXml( System.Xml.XmlReader reader, IDictionary<string, Type> extensions )
        {
            string compressionMode = reader.GetAttribute( "compressionMode" );
            if ( String.IsNullOrEmpty( compressionMode ) )
                throw new Exception( "'compressionMode' attribute expected on the CompressionAssertion in Policy" );
            else
                _compressionMode = ( CompressionMode )Enum.Parse( typeof( CompressionMode ), compressionMode, true );

            string compressionLevel = reader.GetAttribute( "compressionLevel" );
            if ( String.IsNullOrEmpty( compressionLevel ) )
                throw new Exception( "'compressionLevel' attribute expected on the CompressionAssertion in Policy" );
            else
                _compressionLevel = ( CompressionLevel )Enum.Parse( typeof( CompressionLevel ), compressionLevel, true );

            string threshold = reader.GetAttribute( "threshold" );
            if ( String.IsNullOrEmpty( threshold ) )
                throw new Exception( "'threshold' attribute expected on the CompressionAssertion in Policy" );
            else
                _threshold = Convert.ToInt32( threshold, CultureInfo.InvariantCulture );

            reader.Read( );
        }
        
        public override SoapFilter CreateClientInputFilter( FilterCreationContext context )
        {
            return new CompressionInputFilter( );
        }

        public override SoapFilter CreateClientOutputFilter( FilterCreationContext context )
        {
            return new CompressionOutputFilter( _compressionMode, _compressionLevel, _threshold );
        }

        public override SoapFilter CreateServiceInputFilter( FilterCreationContext context )
        {
            return new CompressionInputFilter( );
        }

        public override SoapFilter CreateServiceOutputFilter( FilterCreationContext context )
        {
            return new CompressionOutputFilter( _compressionMode, _compressionLevel, _threshold );
        }
    }
}