using System;

using Microsoft.Web.Services3;

namespace WSCompression
{
    public sealed class CompressionOutputFilter : SoapFilter
    {
        private CompressionMode _compressionMode;
        private CompressionLevel _compressionLevel;
        private int _threshold;

        public CompressionOutputFilter( CompressionMode compressionMode, CompressionLevel compressionLevel, int threshold )
        {
            _compressionMode = compressionMode;
            _compressionLevel = compressionLevel;
            _threshold = threshold;
        }
        
        public override SoapFilterResult ProcessMessage( SoapEnvelope envelope )
        {
            if ( envelope == null )
                throw new ArgumentNullException( "envelope" );

            if ( _compressionMode == CompressionMode.None )
                return SoapFilterResult.Continue;

            int len = envelope.Body.InnerXml.Length;
            if ( len >= _threshold )
            {
                Compression compression = new Compression( _compressionMode, _compressionLevel );
                compression.Compress( envelope );
            }

            return SoapFilterResult.Continue;
        }
    }
}