using System;
using System.Xml;
using System.Collections;

using Microsoft.Web.Services3.Xml;

namespace WSCompression
{
    /// <summary>
    /// List of references. This will link DataReference headers to the compressed data.
    /// </summary>
    public class ReferenceList : IXmlElement
    {
        private ArrayList _refList;

        public ReferenceList( )
        {
            _refList = new ArrayList( );
        }

        public ReferenceList( XmlElement element )
        {
            _refList = new ArrayList( );
            LoadXml( element );
        }

        public static bool IsValidElement( XmlElement element )
        {
            if ( element == null )
                throw new ArgumentNullException( "element" );

            return ( element.LocalName == WSCompression.ElementNames.ReferenceList && element.NamespaceURI == WSCompression.NamespaceURI );
        }

        public void AddReference( string reference )
        {
            if ( String.IsNullOrEmpty( reference ) )
                throw new ArgumentNullException( "reference" );

            if ( reference.StartsWith( "#" ) )
                reference = reference.Substring( 1 );

            _refList.Add( reference );
        }

        public IEnumerator GetEnumerator( )
        {
            return _refList.GetEnumerator( );
        }

        public int Count
        {
            get
            {
                return _refList.Count;
            }
        }

        public string this[ int index ]
        {
            get { return ( string )_refList[ index ]; }
        }

        public static string GenerateNewReference( )
        {
            return "#CompressedContent-" + Guid.NewGuid( ).ToString( "D" );
        }

        #region IXmlElement Members

        public void LoadXml( XmlElement element )
        {
            if ( element == null )
                throw new ArgumentNullException( "element" );

            if ( !IsValidElement( element ) )
                throw new ArgumentException( "Element is not a valid ReferenceList node." );

            foreach ( XmlNode referenceNode in element.ChildNodes )
            {
                if ( referenceNode.LocalName == WSCompression.ElementNames.DataReference &&
                    referenceNode.NamespaceURI == WSCompression.NamespaceURI )
                {
                    XmlElement referenceElement = ( XmlElement )referenceNode;
                    string reference = referenceElement.GetAttribute( WSCompression.AttributeNames.Uri );
                    if ( !String.IsNullOrEmpty( reference ) )
                        AddReference( reference );
                }
            }

        }

        public XmlElement GetXml( XmlDocument document )
        {
            if ( document == null )
                throw new ArgumentNullException( "document" );

            XmlElement referenceList = document.CreateElement( WSCompression.Prefix, WSCompression.ElementNames.ReferenceList, WSCompression.NamespaceURI );
            foreach ( string reference in _refList )
            {
                XmlElement dataReference = document.CreateElement( WSCompression.Prefix, WSCompression.ElementNames.DataReference, WSCompression.NamespaceURI );
                if ( reference.StartsWith( "#" ) )
                {
                    dataReference.SetAttribute( WSCompression.AttributeNames.Uri, reference );
                }
                else
                {
                    dataReference.SetAttribute( WSCompression.AttributeNames.Uri, "#" + reference );
                }
                referenceList.AppendChild( dataReference );
            }
            return referenceList;
        }

        #endregion
    }
}